from typing import Callable

import typer

from thestage.cli_command import CliCommand, CliCommandAvailability
from thestage.color_scheme.color_scheme import ColorScheme
from thestage.config import config_storage
from rich import print


def cli_command(command_id: CliCommand):
    def decorator(func: Callable):
        setattr(func, "__cli_command__", command_id)
        return func
    return decorator


def get_command_metadata(command_id: CliCommand) -> dict:
    return {
        "rich_help_panel": get_command_help_panel(command_id),
        "deprecated": is_command_deprecated(command_id),
    }


def get_command_group_help_panel() -> str:
    return "Command Groups"


def get_command_help_panel(command: CliCommand) -> str:
    if config_storage.APP_CONFIG.runtime.allowed_commands.get(command) == CliCommandAvailability.ALLOWED:
        return "Allowed Commands"
    if config_storage.APP_CONFIG.runtime.allowed_commands.get(command) == CliCommandAvailability.RESTRICTED:
        return "Restricted Commands"
    if config_storage.APP_CONFIG.runtime.allowed_commands.get(command) == CliCommandAvailability.DEPRECATED:
        return "Deprecated Commands"


def is_command_deprecated(command: CliCommand) -> bool:
    if config_storage.APP_CONFIG.runtime.allowed_commands.get(command) == CliCommandAvailability.DEPRECATED:
        return True
    return False


def check_command_permission(executed_command: CliCommand):
    if config_storage.APP_CONFIG.runtime.is_token_valid == False and executed_command != CliCommand.CONFIG_SET:
        print(f"[{ColorScheme.WARNING.value}]Your API Token is not valid. You can update the token using 'thestage config set' command[{ColorScheme.WARNING.value}]")

    is_allowed = config_storage.APP_CONFIG.runtime.allowed_commands.get(executed_command) == CliCommandAvailability.ALLOWED
    if not is_allowed:
        typer.echo("Action is not allowed")
        raise typer.Exit(code=1)



import sys
import traceback

from thestage.helpers.logger.app_logger import app_logger


def exception_handler(exception_type, exception, tb):
    print(exception)
    tb_list = traceback.format_exception(exception_type, exception, tb)
    tb_text = ''.join(tb_list)
    app_logger.error(tb_text)


sys.excepthook = exception_handler



from enum import Enum


class ProviderNameEnumDto(str, Enum):
    Amazon: str = 'Amazon'
    DigitalOcean: str = 'DigitalOcean' # old
    Exoscale: str = 'Exoscale'  # old
    Vultr: str = 'Vultr'    # old
    Nebius: str = 'Nebius'
    Azure: str = 'Azure'
    UNKNOWN: str = 'UNKNOWN'



from typing import Optional, List

from pydantic import Field, ConfigDict, BaseModel

from thestage.services.clients.thestage_api.dtos.entity_filter_request import EntityFilterRequest


class GetInferenceSimulatorRequest(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    slug: Optional[str] = Field(None, alias='slug')


from thestage.helpers import exception_hook
import traceback

from thestage.cli_command_helper import get_command_group_help_panel
from thestage.helpers.logger.app_logger import app_logger, get_log_path_from_os
from thestage.services.service_factory import ServiceFactory
from rich import print


def main():
    service_factory = ServiceFactory()
    config_provider = service_factory.get_config_provider()
    config = config_provider.build_config()

    try:
        try:
            api_client = service_factory.get_thestage_api_client()
            token_info = api_client.validate_token(config.main.thestage_auth_token)
            config_provider.update_allowed_commands_and_is_token_valid(validate_token_response=token_info)
        except Exception as e:
            app_logger.error(f'{traceback.format_exc()}')
            print('Unexpected error occurred')  # TODO inquire what we want here if backend is offline
            print(f'Application logs path: {str(get_log_path_from_os())}')
            return

        from thestage.controllers import base_controller, container_controller, instance_controller, project_controller, \
            config_controller

        base_controller.app.add_typer(project_controller.app, name="project", rich_help_panel=get_command_group_help_panel())
        base_controller.app.add_typer(container_controller.app, name="container", rich_help_panel=get_command_group_help_panel())
        base_controller.app.add_typer(instance_controller.app, name="instance", rich_help_panel=get_command_group_help_panel())
        base_controller.app.add_typer(config_controller.app, name="config", rich_help_panel=get_command_group_help_panel())

        base_controller.app()
    except KeyboardInterrupt:
        print('THESTAGE: Keyboard Interrupt')



from abc import ABC, abstractmethod
from typing import Any, Optional, Tuple


class AbstractMapper(ABC):

    @abstractmethod
    def build_entity(self, item: Any) -> Optional[Any]:
        raise NotImplementedError()



from typing import Optional

from thestage.entities.project_task import ProjectTaskEntity
from thestage.services.abstract_mapper import AbstractMapper
from thestage.services.task.dto.task_dto import TaskDto


class ProjectTaskMapper(AbstractMapper):

    def build_entity(self, item: TaskDto) -> Optional[ProjectTaskEntity]:
        if not item:
            return None

        return ProjectTaskEntity(
            id=item.id or '',
            title=item.title or '',
            status=item.frontend_status.status_translation or '',
            docker_container_id=item.docker_container_id or '',
            docker_container_slug=(str(item.docker_container.slug) or '') if item.docker_container else '',
            started_at=item.started_at or '',
            finished_at=item.finished_at or '',
        )



from datetime import datetime
from typing import Optional, List, Dict

from pydantic import Field, BaseModel, ConfigDict

from thestage.services.clients.thestage_api.dtos.base_response import TheStageBasePaginatedResponse
from thestage.services.clients.thestage_api.dtos.frontend_status import FrontendStatusDto
from thestage.services.clients.thestage_api.dtos.paginated_entity_list import PaginatedEntityList
from thestage.services.clients.thestage_api.dtos.pagination_data import PaginationData
from thestage.services.clients.thestage_api.dtos.instance_detected_gpus import InstanceDetectedGpusDto
from thestage.services.clients.thestage_api.dtos.enums.instance_type import InstanceTypeEnumDto
from thestage.services.clients.thestage_api.dtos.enums.selfhosted_status import SelfhostedBusinessStatus
from thestage.services.clients.thestage_api.dtos.enums.cpu_type import InstanceCpuType


class SelfHostedInstanceDto(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    instance_type: Optional[InstanceTypeEnumDto] = Field(InstanceTypeEnumDto.UNKNOWN, alias='instanceType')
    id: Optional[int] = Field(None, alias='id')
    ram_size_gb: Optional[int] = Field(None, alias='ramSizeGb')
    client_id: Optional[int] = Field(None, alias='clientId')
    slug: Optional[str] = Field(None, alias='slug')
    title: Optional[str] = Field(None, alias='title')
    cpu_type: Optional[InstanceCpuType] = Field(InstanceCpuType.UNKNOWN, alias='cpuType')
    cpu_name: Optional[str] = Field(None, alias='cpuName')
    cpu_cores: Optional[int] = Field(None, alias='cpuCores')
    gpu_count: Optional[int] = Field(None, alias='gpuCount')
    detected_gpus: Optional[InstanceDetectedGpusDto] = Field(None, alias='detectedGpus')
    hardware_profile_id: Optional[str] = Field(None, alias='hardwareProfileId')
    disk_size_gb: Optional[int] = Field(None, alias='diskSizeGb')

    frontend_status: Optional[FrontendStatusDto] = Field(None, alias='frontendStatus')
    business_status: SelfhostedBusinessStatus = Field(SelfhostedBusinessStatus.UNKNOWN, alias='businessStatus')

    updated_at: Optional[datetime] = Field(None, alias='updatedAt')
    created_at: Optional[datetime] = Field(None, alias='createdAt')
    ip_address: Optional[str] = Field(None, alias='ipAddress')
    # not present now
    host_username: Optional[str] = Field(None, alias='hostUsername')


class SelfHostedInstanceListResponse(TheStageBasePaginatedResponse):
    paginated_list: Optional[PaginatedEntityList[SelfHostedInstanceDto]] = Field(None, alias='selfHostedInstanceList')


class SelfHostedRentedItemResponse(TheStageBasePaginatedResponse):
    model_config = ConfigDict(use_enum_values=True)

    selfhosted_instance: Optional[SelfHostedInstanceDto] = Field(None, alias='selfhostedInstance')


class SelfHostedRentedRentedBusinessStatusMapperResponse(TheStageBasePaginatedResponse):
    model_config = ConfigDict(use_enum_values=True)

    selfhosted_instance_business_status_map: Dict[str, str] = Field(default={}, alias='selfhostedInstanceBusinessStatusMap')



from typing import Optional
from pydantic import BaseModel, ConfigDict, Field

class InferenceSimulatorDto(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    id: Optional[int] = Field(None, alias="id")
    client_id: Optional[int] = Field(None, alias="clientId")
    title: Optional[str] = Field(None, alias="title")
    slug: Optional[str] = Field(None, alias="slug")
    docker_container_id: Optional[int] = Field(None, alias="dockerContainerId")
    instance_rented_id: Optional[int] = Field(None, alias="instanceRentedId")
    selfhosted_instance_id: Optional[int] = Field(None, alias="selfhostedInstanceId")
    project_id: Optional[int] = Field(None, alias="projectId")
    status: Optional[str] = Field(None, alias="status")
    commit_hash: Optional[str] = Field(None, alias="commitHash")
    http_endpoint: Optional[str] = Field(None, alias="httpEndpoint")
    grpc_endpoint: Optional[str] = Field(None, alias="grpcEndpoint")
    metrics_endpoint: Optional[str] = Field(None, alias="metricsEndpoint")
    inference_metadata: Optional[str] = Field(None, alias="inferenceMetadata")
    qlip_serve_metadata: Optional[str] = Field(None, alias="qlipServeMetadata")
    created_at: Optional[str] = Field(None, alias="createdAt")
    updated_at: Optional[str] = Field(None, alias="updatedAt")


from typing import Optional

from pydantic import BaseModel, Field


class UserProfile(BaseModel):
    email: Optional[str] = Field(None, alias='email')



class UserProfileResponse(BaseModel):
    userProfile: Optional[UserProfile] = Field(None, alias='userProfile')



from thestage import __app_name__

from thestage.controllers import base_controller, container_controller, instance_controller, \
    config_controller, project_controller

base_controller.app.add_typer(container_controller.app, name="container")
base_controller.app.add_typer(instance_controller.app, name="instance")
base_controller.app.add_typer(config_controller.app, name="config")
base_controller.app.add_typer(project_controller.app, name="project")


def main():
    # example of a command to debug
    project_controller.app([
        "run",
        "-wd",
        "/Users/alexey/Documents/clonetest",
        "-dcuid",
        '62-1',
        '-t',
        'task',
        '-com',
        "'echo 123'"
    ], prog_name=__app_name__)


if __name__ == "__main__":
    main()



from pathlib import Path
from typing import Optional, List

from thestage.cli_command import CliCommand
from thestage.cli_command_helper import get_command_group_help_panel, get_command_metadata, check_command_permission
from thestage.helpers.logger.app_logger import app_logger
from thestage.services.instance.instance_service import InstanceService
from thestage.i18n.translation import __
from thestage.controllers.utils_controller import \
    validate_config_and_get_service_factory, get_current_directory

import typer


app = typer.Typer(no_args_is_help=True, help=__("Manage server instances"))

rented = typer.Typer(no_args_is_help=True, help=__("Manage rented server instances"))
self_hosted = typer.Typer(no_args_is_help=True, help=__("Manage self-hosted instances"))

app.add_typer(rented, name="rented", rich_help_panel=get_command_group_help_panel())
app.add_typer(self_hosted, name="self-hosted", rich_help_panel=get_command_group_help_panel())


@rented.command(name="ls", help=__("List rented server instances"), **get_command_metadata(CliCommand.INSTANCE_RENTED_LS))
def rented_list(
        row: int = typer.Option(
            5,
            '--row',
            '-r',
            help=__("Set number of rows displayed per page"),
            is_eager=False,
        ),
        page: int = typer.Option(
            1,
            '--page',
            '-p',
            help=__("Set starting page for displaying output"),
            is_eager=False,
        ),
        statuses: List[str] = typer.Option(
            None,
            '--status',
            '-s',
            help=__("Filter by status, use --status all to list all rented server instances"),
            is_eager=False,
        ),
):
    command_name = CliCommand.INSTANCE_RENTED_LS
    app_logger.info(f'Running {command_name} from {get_current_directory()}')
    check_command_permission(command_name)

    service_factory = validate_config_and_get_service_factory()
    instance_service: InstanceService = service_factory.get_instance_service()

    instance_service.print_rented_instance_list(
        statuses=statuses,
        row=row,
        page=page
    )

    typer.echo(__("Rented server instances listing complete"))
    raise typer.Exit(0)


@rented.command(name="connect", no_args_is_help=True, help=__("Connect to rented server instance"), **get_command_metadata(CliCommand.INSTANCE_RENTED_CONNECT))
def instance_connect(
        instance_uid: Optional[str] = typer.Argument(
            help=__("Rented server instance unique ID"),
        ),
        private_ssh_key_path: str = typer.Option(
            None,
            "--private-key-path",
            "-pk",
            help=__("Path to private key that will be accepted by remote server (optional)"),
            is_eager=False,
        )
):
    command_name = CliCommand.INSTANCE_RENTED_CONNECT
    app_logger.info(f'Running {command_name} from {get_current_directory()}')
    check_command_permission(command_name)

    if not instance_uid:
        typer.echo(__('Rented server instance unique ID is required'))
        raise typer.Exit(1)

    if private_ssh_key_path and not Path(private_ssh_key_path).is_file():
        typer.echo(f'No file found at provided path {private_ssh_key_path}')
        raise typer.Exit(1)

    service_factory = validate_config_and_get_service_factory()
    instance_service: InstanceService = service_factory.get_instance_service()

    instance_service.connect_to_rented_instance(
        instance_rented_slug=instance_uid,
        input_ssh_key_path=private_ssh_key_path
    )

    app_logger.info(f'Stop connecting to rented server instance')
    raise typer.Exit(0)


@self_hosted.command(name="ls", help=__("List self-hosted instances"), **get_command_metadata(CliCommand.INSTANCE_SELF_HOSTED_LS))
def self_hosted_list(
        row: int = typer.Option(
            5,
            '--row',
            '-r',
            help=__("Set number of rows displayed per page"),
            is_eager=False,
        ),
        page: int = typer.Option(
            1,
            '--page',
            '-p',
            help=__("Set starting page for displaying output"),
            is_eager=False,
        ),
        statuses: List[str] = typer.Option(
            None,
            '--status',
            '-s',
            help=__("Filter by status, use --status all to list all self-hosted server instances"),
            is_eager=False,
        ),
):
    command_name = CliCommand.INSTANCE_SELF_HOSTED_LS
    app_logger.info(f'Running {command_name} from {get_current_directory()}')
    check_command_permission(command_name)

    service_factory = validate_config_and_get_service_factory()
    instance_service: InstanceService = service_factory.get_instance_service()

    instance_service.print_self_hosted_instance_list(
        statuses=statuses,
        row=row,
        page=page
    )

    typer.echo(__("Self-hosted instances listing complete"))
    raise typer.Exit(0)


@self_hosted.command(name="connect", no_args_is_help=True, help=__("Connect to self-hosted instance"), **get_command_metadata(CliCommand.INSTANCE_SELF_HOSTED_CONNECT))
def self_hosted_connect(
        instance_slug: Optional[str] = typer.Argument(help=__("Self-hosted instance unique ID"),),
        username: Optional[str] = typer.Option(
            None,
            '--username',
            '-u',
            help=__("Username for server instance (required when connecting to self-hosted instance)"),
            is_eager=False,
        ),
        private_ssh_key_path: str = typer.Option(
            None,
            "--private-key-path",
            "-pk",
            help=__("Path to private key that will be accepted by remote server (optional)"),
            is_eager=False,
        ),
):
    command_name = CliCommand.INSTANCE_SELF_HOSTED_CONNECT
    app_logger.info(f'Running {command_name} from {get_current_directory()}')
    check_command_permission(command_name)

    if not instance_slug:
        typer.echo(__('Self-hosted instance unique ID is required'))
        raise typer.Exit(1)

    if private_ssh_key_path and not Path(private_ssh_key_path).is_file():
        typer.echo(f'No file found at provided path {private_ssh_key_path}')
        raise typer.Exit(1)

    service_factory = validate_config_and_get_service_factory()
    instance_service: InstanceService = service_factory.get_instance_service()

    instance_service.connect_to_selfhosted_instance(
        selfhosted_instance_slug=instance_slug,
        username=username,
        input_ssh_key_path=private_ssh_key_path
    )

    app_logger.info(f'Stop connecting to self-hosted instance')
    raise typer.Exit(0)



from enum import Enum


class DockerContainerAction(str, Enum):
    CREATE: str = 'CREATE'
    UPDATE: str = 'UPDATE'
    START: str = 'START'
    RESTART: str = 'RESTART'
    STOP: str = 'STOP'
    DELETE: str = 'DELETE'



from typing import Optional

from pydantic import BaseModel, ConfigDict, Field


class ProjectInferenceSimulatorModelEntity(BaseModel):

    model_config = ConfigDict(
        populate_by_name=True,
        use_enum_values=True,
    )

    id: Optional[int] = Field(None, alias='ID')
    slug: Optional[str] = Field(None, alias='UNIQUE_ID')
    status: Optional[str] = Field(None, alias='STATUS')
    commit_hash: Optional[str] = Field(None, alias='COMMIT_HASH')
    environment_metadata: Optional[str] = Field(None, alias='ENVIRONMENT_METADATA')


from typing import Optional, List

from pydantic import Field

from thestage.entities.file_item import FileItemEntity


class SftpFileItemEntity(FileItemEntity):

    instance_path: Optional[str] = Field(None)
    container_path: Optional[str] = Field(None)
    children: List['SftpFileItemEntity'] = Field(default=[])
    dest_path: Optional[str] = Field(None)



from typing import Optional, List

from pydantic import Field, ConfigDict, BaseModel

from thestage.services.clients.thestage_api.dtos.entity_filter_request import EntityFilterRequest
from thestage.services.project.dto.inference_simulator_dto import InferenceSimulatorDto


class GetInferenceSimulatorResponse(BaseModel):
    model_config = ConfigDict(use_enum_values=True)
    inferenceSimulator: Optional[InferenceSimulatorDto] = Field(None, alias='inferenceSimulator')





from enum import Enum


class InstanceRentedBusinessStatus(str, Enum):
    IN_QUEUE: str = 'IN_QUEUE'
    CREATING: str = 'CREATING'
    ONLINE: str = 'ONLINE'
    TERMINATING: str = 'TERMINATING'
    STOPPED: str = 'STOPPED'
    STOPPING: str = 'STOPPING'
    STARTING: str = 'STARTING'
    REBOOTING: str = 'REBOOTING'
    UNREACHABLE_DAEMON: str = 'UNREACHABLE_DAEMON'
    DELETED: str = 'DELETED'
    RENTAL_ERROR: str = 'RENTAL_ERROR'
    UNKNOWN: str = 'UNKNOWN'
    ALL: str = 'ALL'



from typing import Dict

from pydantic import Field, ConfigDict

from thestage.services.clients.thestage_api.dtos.base_response import TheStageBasePaginatedResponse


class InferenceSimulatorStatusMapperResponse(TheStageBasePaginatedResponse):
    model_config = ConfigDict(use_enum_values=True)

    inference_simulator_status_map: Dict[str, str] = Field(default={}, alias='inferenceSimulatorStatusMap')



from typing import Optional

from pydantic import Field, ConfigDict, BaseModel


class ProjectStartInferenceSimulatorRequest(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    projectSlug: str = Field(None, alias='projectSlug')
    instanceRentedUId: Optional[str] = Field(None, alias='instanceRentedUId')
    selfhostedInstanceUId: Optional[str] = Field(None, alias='selfhostedInstanceUId')
    commitHash: Optional[str] = Field(None, alias='commitHash')
    slug: Optional[str] = Field(None, alias='slug')
    inferenceDir: Optional[str] = Field(None, alias='inferenceDir')
    isSkipInstallation: Optional[bool] = Field(False, alias='isSkipInstallation')


from pathlib import Path
from typing import Optional

from thestage.cli_command import CliCommand
from thestage.cli_command_helper import get_command_metadata, check_command_permission
from thestage.i18n.translation import __
from thestage.helpers.logger.app_logger import app_logger
from thestage.controllers.utils_controller import get_current_directory, validate_config_and_get_service_factory
from thestage import __app_name__, __version__

import typer

from thestage.services.connect.connect_service import ConnectService

app = typer.Typer(no_args_is_help=True,)


@app.command(name='version', help="Get application's name and version", no_args_is_help=False, **get_command_metadata(CliCommand.VERSION))
def version():
    command_name = CliCommand.VERSION
    app_logger.info(f'Running {command_name} from {get_current_directory()}')
    check_command_permission(command_name)

    typer.echo(f"{__app_name__} v{__version__}")
    raise typer.Exit(0)


@app.command(name="connect", no_args_is_help=True, help=__("Connect to server instance or container or task"), **get_command_metadata(CliCommand.CONNECT))
def connect(
        uid: Optional[str] = typer.Argument(
            help=__("Unique ID of server instance or container or task ID"), ),
        username: Optional[str] = typer.Option(
            None,
            '--username',
            '-u',
            help=__("Username for the server instance (required when connecting to self-hosted instance)"),
            is_eager=False,
        ),
        private_ssh_key_path: str = typer.Option(
            None,
            "--private-key-path",
            "-pk",
            help=__("Path to private key that will be accepted by remote server (optional)"),
            is_eager=False,
        ),
):
    command_name = CliCommand.CONNECT
    app_logger.info(f'Running {command_name} from {get_current_directory()}')
    check_command_permission(command_name)

    if private_ssh_key_path and not Path(private_ssh_key_path).is_file():
        typer.echo(f'No file found at provided path {private_ssh_key_path}')
        raise typer.Exit(1)

    service_factory = validate_config_and_get_service_factory()

    connect_service: ConnectService = service_factory.get_connect_service()

    connect_service.connect_to_entity(
        uid=uid,
        username=username,
        private_key_path=private_ssh_key_path
    )


    app_logger.info(f'Stop connect to entity')
    raise typer.Exit(0)



from git import RemoteProgress


class ProgressPrinter(RemoteProgress):
    def __init__( self ):
        super().__init__()

        self.__all_dropped_lines = []

    def update( self, op_code, cur_count, max_count=None, message='' ):
        pass

    def line_dropped( self, line ):
        if line.startswith( 'POST git-upload-pack' ):
            return

        self.__all_dropped_lines.append( line )

    def allErrorLines( self ):
        return self.error_lines + self.__all_dropped_lines

    def allDroppedLines( self ):
        return self.__all_dropped_lines


from typing import Optional

from pydantic import ConfigDict, BaseModel, Field


class FrontendStatusDto(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    status_key: Optional[str] = Field(None, alias='statusKey')
    status_translation: Optional[str] = Field(None, alias='statusTranslation')



from abc import ABC
from typing import List, Dict, Any, Optional

import typer
from tabulate import tabulate
from thestage.i18n.translation import __
from thestage.services.abstract_mapper import AbstractMapper
from thestage.services.clients.thestage_api.dtos.paginated_entity_list import PaginatedEntityList


class AbstractService(ABC):
    # TODO remove recursion
    def print(
            self,
            func_get_data,
            func_special_params: Dict[str, Any],
            mapper: AbstractMapper,
            headers: List[str],
            row: int = 5,
            page: int = 1,
            show_index: str = 'always',
            max_col_width: List[Optional[int]] = None,
            depth: int = 0,
    ):
        paginated_entity_list: PaginatedEntityList = func_get_data(
            row=row,
            page=page,
            **func_special_params,
        )

        result = list(map(lambda x: mapper.build_entity(x), paginated_entity_list.entities))

        if result:
            raw_data = [list(item.model_dump(
                by_alias=True,
            ).values()) for item in result]
        else:
            if depth == 0:
                typer.echo(__("No items found"))
            else:
                typer.echo(__("Listing completed"))
            raise typer.Exit(0)

        # tabulate() crashes on any None value, check mappers

        typer.echo(__(
            "Current page: %page% | Total pages: %total_pages% | Items per page: %limit%",
            {
                'page': str(page),
                'limit': str(row),
                'total_pages': str(paginated_entity_list.pagination_data.total_pages),
            }
        ))

        typer.echo(tabulate(
            raw_data,
            headers=headers,
            showindex=show_index,
            tablefmt="double_grid",
            maxcolwidths=max_col_width,
        ))

        if page >= paginated_entity_list.pagination_data.total_pages:
            typer.echo(__("Listing completed"))
            raise typer.Exit(0)

        next_page: int = typer.prompt(
            text=__('Go to next page (0 to exit)?'),
            default=page + 1,
            show_choices=False,
            type=int,
            show_default=True,
        )
        if next_page == 0:
            raise typer.Exit(0)
        else:
            self.print(
                func_get_data=func_get_data,
                func_special_params=func_special_params,
                mapper=mapper,
                headers=headers,
                row=row,
                page=next_page,
                max_col_width=max_col_width,
                show_index=show_index,
                depth=depth + 1,
            )



from typing import Optional

from pydantic import BaseModel, Field, ConfigDict

from thestage.services.clients.thestage_api.dtos.enums.location_region import LocationRegionEnumDto
from thestage.services.clients.thestage_api.dtos.enums.provider_name import ProviderNameEnumDto


class CloudProviderRegionDto(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    id: Optional[int] = Field(None, alias='id')
    region_key: Optional[str] = Field(None, alias='regionKey')
    provider_name: ProviderNameEnumDto = Field(ProviderNameEnumDto.UNKNOWN, alias='providerName')
    location_region: LocationRegionEnumDto = Field(LocationRegionEnumDto.UNKNOWN, alias='locationRegion')
    location_country: Optional[str] = Field(None, alias='locationCountry')
    location_detail: Optional[str] = Field(None, alias='locationDetail')
    is_active_for_storages: Optional[bool] = Field(False, alias='isActiveForStorages')
    is_active_for_instances: Optional[bool] = Field(False, alias='isActiveForInstances')
